@@ -4,7 +4,9 @@ urlpatterns = patterns('graphite.dashboard.views',
   ('^save/(?P<name>[^/]+)', 'save'),
   ('^load/(?P<name>[^/]+)', 'load'),
   ('^delete/(?P<name>[^/]+)', 'delete'),
+  ('^create-temporary/?', 'create_temporary'),
   ('^find/', 'find'),
+  ('^help/', 'help'),
   ('^(?P<name>[^/]+)', 'dashboard'),
   ('', 'dashboard'),
 )
