@@ -18,6 +18,7 @@ import time
 from django.conf import settings
 from graphite.logger import log
 from graphite.storage import STORE, LOCAL_STORE
+from graphite.render.hashing import ConsistentHashRing
 
 try:
   import cPickle as pickle
@@ -92,100 +93,115 @@ class TimeSeries(list):
 
 
 class CarbonLinkPool:
-  def __init__(self,hosts,timeout):
-    self.hosts = hosts
+  def __init__(self, hosts, timeout):
+    self.hosts = [ (server, instance) for (server, port, instance) in hosts ]
+    self.ports = dict( ((server, instance), port) for (server, port, instance) in hosts )
     self.timeout = float(timeout)
+    self.hash_ring = ConsistentHashRing(self.hosts)
     self.connections = {}
+    self.last_failure = {}
     # Create a connection pool for each host
-    for host in hosts:
+    for host in self.hosts:
       self.connections[host] = set()
 
-  def selectHost(self, metric):
+  def select_host(self, metric):
     "Returns the carbon host that has data for the given metric"
-    return self.hosts[ hash(metric) % len(self.hosts) ]
+    return self.hash_ring.get_node(metric)
 
-  def getConnection(self, host):
+  def get_connection(self, host):
     # First try to take one out of the pool for this host
+    (server, instance) = host
+    port = self.ports[host]
     connectionPool = self.connections[host]
     try:
       return connectionPool.pop()
     except KeyError:
       pass #nothing left in the pool, gotta make a new connection
 
-    log.cache("CarbonLink creating a new socket for %s:%d" % host)
+    log.cache("CarbonLink creating a new socket for %s" % str(host))
     connection = socket.socket()
     connection.settimeout(self.timeout)
-    connection.connect(host)
-    connection.setsockopt( socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1 )
-    return connection
-
-  def putConnectionInPool(self, host, connection):
-    connectionPool = self.connections[host]
-    connectionPool.add(connection)
-
-  def removeConnectionFromPool(self, host, connection):
-    connectionPool = self.connections.get(host, set())
-    connectionPool.discard(connection)
-
-  def sendRequest(self, metric):
-    "Sends a request and returns a completion callback"
-    host = self.selectHost(metric)
-    query = struct.pack("!L", len(metric)) + metric # 32-bit length prefix string
-    connection = None
-
     try:
-      connection = self.getConnection(host)
-      connection.sendall(query)
-
-      # To keep things asynchronous we return a result callback
-      def receiveResponse():
-        try:
-          buf = ''
-          remaining = 4
-          message_size = None
-
-          while remaining:
-            packet = connection.recv(remaining)
-            assert packet, "CarbonLink lost connection to %s:%d" % host
-
-            buf += packet
-
-            if message_size is None:
-              if len(buf) == 4:
-                remaining = message_size = struct.unpack("!L", buf)[0]
-                buf = ''
-                continue
-
-            remaining -= len(packet)
-
-          # We're done with the connection for this request, put it in the pool
-          self.putConnectionInPool(host, connection)
-
-          # Now parse the response
-          points = pickle.loads(buf)
-          log.cache("CarbonLink to %s, retrieved %d points for %s" % (host,len(points),metric))
-
-          for point in points:
-            yield point
-
-        except:
-          log.exception("CarbonLink to %s, exception while getting response" % str(host))
-          self.removeConnectionFromPool(host, connection)
-
-      return receiveResponse
+      connection.connect( (server, port) )
     except:
-      log.exception("CarbonLink to %s, exception while sending request" % str(host))
-      if connection:
-        self.removeConnectionFromPool(host, connection)
-      noResults = lambda: []
-      return noResults
+      self.last_failure[host] = time.time()
+      raise
+    else:
+      connection.setsockopt( socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1 )
+      return connection
 
+  def query(self, metric):
+    request = dict(type='cache-query', metric=metric)
+    results = self.send_request(request)
+    log.cache("CarbonLink cache-query request for %s returned %d datapoints" % (metric, len(results)))
+    return results['datapoints']
+
+  def get_metadata(self, metric, key):
+    request = dict(type='get-metadata', metric=metric, key=key)
+    results = self.send_request(request)
+    log.cache("CarbonLink get-metadata request received for %s:%s" % (metric, key))
+    return results['value']
+
+  def set_metadata(self, metric, key, value):
+    request = dict(type='set-metadata', metric=metric, key=key, value=value)
+    results = self.send_request(request)
+    log.cache("CarbonLink set-metadata request received for %s:%s" % (metric, key))
+    return results
+
+  def send_request(self, request):
+    metric = request['metric']
+    serialized_request = pickle.dumps(request, protocol=-1)
+    len_prefix = struct.pack("!L", len(serialized_request))
+    request_packet = len_prefix + serialized_request
+
+    host = self.select_host(metric)
+    conn = self.get_connection(host)
+    try:
+      conn.sendall(request_packet)
+      result = self.recv_response(conn)
+    except:
+      self.last_failure[host] = time.time()
+      raise
+    else:
+      self.connections[host].add(conn)
+      if 'error' in result:
+        raise CarbonLinkRequestError(result['error'])
+      else:
+        return result
+
+  def recv_response(self, conn):
+    len_prefix = recv_exactly(conn, 4)
+    body_size = struct.unpack("!L", len_prefix)[0]
+    body = recv_exactly(conn, body_size)
+    return pickle.loads(body)
+
+
+# Utilities
+class CarbonLinkRequestError(Exception):
+  pass
+
+def recv_exactly(conn, num_bytes):
+  buf = ''
+  while len(buf) < num_bytes:
+    data = conn.recv( num_bytes - len(buf) )
+    if not data:
+      raise Exception("Connection lost")
+    buf += data
+
+  return buf
 
 #parse hosts from local_settings.py
 hosts = []
 for host in settings.CARBONLINK_HOSTS:
-  server,port = host.split(':',1)
-  hosts.append( (server,int(port)) )
+  parts = host.split(':')
+  server = parts[0]
+  port = int( parts[1] )
+  if len(parts) > 2:
+    instance = parts[2]
+  else:
+    instance = None
+
+  hosts.append( (server, int(port), instance) )
 
 
 #A shared importable singleton
@@ -208,9 +224,13 @@ def fetchData(requestContext, pathExpr):
 
   for dbFile in store.find(pathExpr):
     log.metric_access(dbFile.metric_path)
-    getCacheResults = CarbonLink.sendRequest(dbFile.real_metric)
     dbResults = dbFile.fetch( timestamp(startTime), timestamp(endTime) )
-    results = mergeResults(dbResults, getCacheResults())
+    try:
+      cachedResults = CarbonLink.query(dbFile.real_metric)
+      results = mergeResults(dbResults, cachedResults)
+    except:
+      log.exception()
+      results = dbResults
 
     if not results:
       continue
