@@ -1,4 +1,5 @@
 """Copyright 2008 Orbitz WorldWide
+   Copyright 2011 Chris Davis
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -12,7 +13,13 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
 
-import md5, time
+from graphite.logger import log
+import time
+try:
+  from hashlib import md5
+except ImportError:
+  from md5 import md5
+import bisect
 
 def hashRequest(request):
   # Normalize the request parameters so ensure we're deterministic
@@ -46,6 +53,38 @@ def stripControlChars(string):
 
 
 def compactHash(string):
-  hash = md5.md5()
+  hash = md5()
   hash.update(string)
   return hash.hexdigest()
+
+
+
+class ConsistentHashRing:
+  def __init__(self, nodes, replica_count=100):
+    self.ring = []
+    self.replica_count = replica_count
+    for node in nodes:
+      self.add_node(node)
+
+  def compute_ring_position(self, key):
+    big_hash = md5( str(key) ).hexdigest()
+    small_hash = int(big_hash[:4], 16)
+    return small_hash
+
+  def add_node(self, key):
+    for i in range(self.replica_count):
+      replica_key = "%s:%d" % (key, i)
+      position = self.compute_ring_position(replica_key)
+      entry = (position, key)
+      bisect.insort(self.ring, entry)
+
+  def remove_node(self, key):
+    self.ring = [entry for entry in self.ring if entry[1] != key]
+
+  def get_node(self, key):
+    position = self.compute_ring_position(key)
+    search_entry = (position, None)
+    index = bisect.bisect_left(self.ring, search_entry)
+    index %= len(self.ring)
+    entry = self.ring[index]
+    return entry[1]
