@@ -16,23 +16,26 @@ from time import time, strftime, localtime
 from datetime import datetime, timedelta
 from random import shuffle
 from httplib import CannotSendRequest
-from urlparse import urlsplit
+from urllib import urlencode
+from urlparse import urlsplit, urlunsplit
+from cgi import parse_qs
 from cStringIO import StringIO
 try:
   import cPickle as pickle
 except ImportError:
   import pickle
 
-from graphite.util import getProfileByUsername
+from graphite.util import getProfileByUsername, json
 from graphite.remote_storage import HTTPConnectionWithTimeout
 from graphite.logger import log
 from graphite.render.evaluator import evaluateTarget
 from graphite.render.attime import parseATTime
 from graphite.render.functions import PieFunctions
-from graphite.render.glyph import GraphTypes
 from graphite.render.hashing import hashRequest, hashData
+from graphite.render.glyph import GraphTypes
 
-from django.http import HttpResponse, HttpResponseServerError
+from django.http import HttpResponse, HttpResponseServerError, HttpResponseRedirect
+from django.template import Context, loader
 from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist
 from django.conf import settings
@@ -42,6 +45,7 @@ def renderView(request):
   start = time()
   (graphOptions, requestOptions) = parseOptions(request)
   useCache = 'noCache' not in requestOptions
+  cacheTimeout = requestOptions['cacheTimeout']
   requestContext = {
     'startTime' : requestOptions['startTime'],
     'endTime' : requestOptions['endTime'],
@@ -103,7 +107,7 @@ def renderView(request):
         data.extend(seriesList)
 
     if useCache:
-      cache.set(dataKey, data)
+      cache.set(dataKey, data, cacheTimeout)
 
     # If data is all we needed, we're done
     if 'pickle' in requestOptions:
@@ -114,7 +118,8 @@ def renderView(request):
       log.rendering('Total pickle rendering time %.6f' % (time() - start))
       return response
 
-    if requestOptions.get('format') == 'csv':
+    format = requestOptions.get('format')
+    if format == 'csv':
       response = HttpResponse(mimetype='text/csv')
       writer = csv.writer(response, dialect='excel')
 
@@ -125,7 +130,25 @@ def renderView(request):
 
       return response
 
-    if 'rawData' in requestOptions:
+    if format == 'json':
+      series_data = []
+      for series in data:
+        timestamps = range(series.start, series.end, series.step)
+        datapoints = zip(series, timestamps)
+        series_data.append( dict(target=series.name, datapoints=datapoints) )
+
+      if 'jsonp' in requestOptions:
+        response = HttpResponse(
+          content="%s(%s)" % (requestOptions['jsonp'], json.dumps(series_data)),
+          mimetype='text/javascript')
+      else:
+        response = HttpResponse(content=json.dumps(series_data), mimetype='application/json')
+
+      response['Pragma'] = 'no-cache'
+      response['Cache-Control'] = 'no-cache'
+      return response
+
+    if format == 'raw':
       response = HttpResponse(mimetype='text/plain')
       for series in data:
         response.write( "%s,%d,%d,%d|" % (series.name, series.start, series.end, series.step) )
@@ -145,7 +168,7 @@ def renderView(request):
   response = buildResponse(image)
 
   if useCache:
-    cache.set(requestKey, response)
+    cache.set(requestKey, response, cacheTimeout)
 
   log.rendering('Total rendering time %.6f seconds' % (time() - start))
   return response
@@ -166,6 +189,7 @@ def parseOptions(request):
   requestOptions['graphType'] = graphType
   requestOptions['graphClass'] = graphClass
   requestOptions['pieMode'] = queryParams.get('pieMode', 'average')
+  requestOptions['cacheTimeout'] = int( queryParams.get('cacheTimeout', settings.DEFAULT_CACHE_DURATION) )
   requestOptions['targets'] = []
   for target in queryParams.getlist('target'):
     if target.lower().startswith('graphite.'): #Strip leading "Graphite." as a convenience
@@ -175,9 +199,11 @@ def parseOptions(request):
   if 'pickle' in queryParams:
     requestOptions['pickle'] = True
   if 'rawData' in queryParams:
-    requestOptions['rawData'] = True
+    requestOptions['format'] = 'raw'
   if 'format' in queryParams:
     requestOptions['format'] = queryParams['format']
+    if 'jsonp' in queryParams:
+      requestOptions['jsonp'] = queryParams['jsonp']
   if 'noCache' in queryParams:
     requestOptions['noCache'] = True
 
@@ -200,16 +226,17 @@ def parseOptions(request):
   # Get the time interval for time-oriented graph types
   if graphType == 'line' or graphType == 'pie':
     if 'until' in queryParams:
-      endTime = parseATTime( queryParams['until'] )
+      untilTime = parseATTime( queryParams['until'] )
     else:
-      endTime = datetime.now()
+      untilTime = datetime.now()
     if 'from' in queryParams:
-      startTime = parseATTime( queryParams['from'] )
+      fromTime = parseATTime( queryParams['from'] )
     else:
-      startTime = endTime - timedelta(days=1)
-    if endTime > datetime.now():
-      endTime = datetime.now()
-    assert startTime < endTime, "Invalid time range!"
+      fromTime = untilTime - timedelta(days=1)
+
+    startTime = min(fromTime, untilTime)
+    endTime = max(fromTime, untilTime)
+    assert startTime != endTime, "Invalid empty time range"
     
     requestOptions['startTime'] = startTime
     requestOptions['endTime'] = endTime
@@ -243,9 +270,9 @@ def delegateRendering(graphType, graphOptions):
       except CannotSendRequest:
         connection = HTTPConnectionWithTimeout(server) #retry once
         connection.timeout = settings.REMOTE_RENDER_CONNECT_TIMEOUT
-        conn.request('POST', '/render/local/', postData)
+        connection.request('POST', '/render/local/', postData)
       # Read the response
-      response = conn.getresponse()
+      response = connection.getresponse()
       assert response.status == 200, "Bad response code %d from %s" % (response.status,server)
       contentType = response.getheader('Content-Type')
       imageData = response.read()
@@ -281,19 +308,28 @@ def renderLocalView(request):
 
 def renderMyGraphView(request,username,graphName):
   profile = getProfileByUsername(username)
-  assert profile, "No such user '%s'" % username
+  if not profile:
+    return errorPage("No such user '%s'" % username)
   try:
     graph = profile.mygraph_set.get(name=graphName)
   except ObjectDoesNotExist:
-    assert False, "User %s doesn't have a MyGraph named '%s'" % (username,graphName)
-  (proto,host,path,query,frag) = urlsplit( graph.url )
-  if query: path += '?' + query
-  conn = HTTPConnectionWithTimeout( host )
-  conn.request('GET', path)
-  resp = conn.getresponse()
-  assert resp.status == 200, "Failed to retrieve image from URL %s" % graph.url
-  imageData = resp.read()
-  return buildResponse(imageData)
+    return errorPage("User %s doesn't have a MyGraph named '%s'" % (username,graphName))
+
+  request_params = dict(request.REQUEST.items())
+  if request_params:
+    url_parts = urlsplit(graph.url)
+    query_string = url_parts[3]
+    if query_string:
+      url_params = parse_qs(query_string)
+      for param, value in url_params.items():
+        if isinstance(value, list):
+          url_params[param] = value[0]
+      url_params.update(request_params)
+      query_string = urlencode(url_params)
+    url = urlunsplit(url_parts[:3] + (query_string,) + url_parts[4:])
+  else:
+    url = graph.url
+  return HttpResponseRedirect(url)
 
 
 def doImageRender(graphClass, graphOptions):
@@ -312,3 +348,9 @@ def buildResponse(imageData):
   response['Cache-Control'] = 'no-cache'
   response['Pragma'] = 'no-cache'
   return response
+
+
+def errorPage(message):
+  template = loader.get_template('500.html')
+  context = Context(dict(message=message))
+  return HttpResponseServerError( template.render(context) )
