@@ -12,14 +12,20 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
 
+from django import VERSION
 from django.conf.urls.defaults import *
 from django.conf import settings
 from django.contrib import admin
 
 admin.autodiscover()
 
+if VERSION[:2] == (1, 0):
+  admin_urls = admin.site.root
+else:
+  admin_urls = include(admin.site.urls)
+
 urlpatterns = patterns('',
-  ('^admin/', include(admin.site.urls)),
+  ('^admin/', admin_urls),
   ('^render/?', include('graphite.render.urls')),
   ('^cli/?', include('graphite.cli.urls')),
   ('^composer/?', include('graphite.composer.urls')),
@@ -29,7 +35,10 @@ urlpatterns = patterns('',
   ('^dashboard/?', include('graphite.dashboard.urls')),
   ('^whitelist/?', include('graphite.whitelist.urls')),
   ('^content/(?P<path>.*)$', 'django.views.static.serve', {'document_root' : settings.CONTENT_DIR}),
-  ('', include('graphite.browser.urls')),
+  ('graphlot/', include('graphite.graphlot.urls')),
+  ('^version/', include('graphite.version.urls')),
+  ('^events/', include('graphite.events.urls')),
+  ('', 'graphite.browser.views.browser'),
 )
 
 handler500 = 'graphite.views.server_error'
